{
 "cells": [
  {
   "cell_type": "markdown",
   "metadata": {},
   "source": [
    "Hurricane Tracker with NHC Data\n",
    "===============================\n",
    "\n",
    "By: Aodhan Sweeney\n",
    "\n",
    "This program is a recreation of the 2014 hur_tracker.py\n",
    "originally written by Unidata Intern Florita Rodriguez. The\n",
    "2019 version comes with updated interface and functionality,\n",
    "as well as changing certain dependencies."
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 1,
   "metadata": {},
   "outputs": [],
   "source": [
    "import gzip\n",
    "from io import BytesIO\n",
    "from io import StringIO"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 2,
   "metadata": {},
   "outputs": [],
   "source": [
    "import cartopy.crs as ccrs\n",
    "import ipywidgets as widgets\n",
    "import matplotlib.pyplot as plt\n",
    "import numpy as np\n",
    "from pandas import DataFrame\n",
    "import requests"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 3,
   "metadata": {},
   "outputs": [],
   "source": [
    "def readUrlFile(url):\n",
    "    \"\"\"readUrlFile is a function created to read a .dat file from a given url\n",
    "    and compile it into a list of strings with given headers.\"\"\"\n",
    "\n",
    "    headers = {'User-agent': 'Unidata Python Client Test'}\n",
    "    response = requests.get(url, headers=headers)\n",
    "    # Store data response in a string buffer\n",
    "    string_buffer = StringIO(response.text)\n",
    "    # Read from the string buffer as if it were a physical file\n",
    "    data = string_buffer.getvalue()\n",
    "    return data.splitlines()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 4,
   "metadata": {},
   "outputs": [],
   "source": [
    "def readGZFile(url):\n",
    "    \"\"\"readGZFile is a function which opens and reads zipped files. In this case it takes in a\n",
    "    .gzfile containing information on each storm and returns a byte buffer split based on\n",
    "    lines.\"\"\"\n",
    "\n",
    "    headers = {'User-agent': 'Unidata Python Client Test'}\n",
    "    response = requests.get(url, headers=headers)\n",
    "    # Store data response in a bytes buffer\n",
    "    bio_buffer = BytesIO(response.content)\n",
    "    # Read from the string buffer as if it were a physical file\n",
    "    gzf = gzip.GzipFile(fileobj=bio_buffer)\n",
    "    data = gzf.read()\n",
    "    return data.splitlines()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 5,
   "metadata": {},
   "outputs": [],
   "source": [
    "def split_storm_info(storm_list):\n",
    "    \"\"\"split_storm_info takes a list of strings and creates a pandas dataframe\n",
    "    for the data set taken off the NHC archive. This function is called in the main to\n",
    "    find all storms.\"\"\"\n",
    "\n",
    "    name, cycloneNum, year, stormType, basin, filename = [], [], [], [], [], []\n",
    "    for line in storm_list[1:]:\n",
    "        fields = line.split(',')\n",
    "        name.append(fields[0].strip())\n",
    "        basin.append(fields[1].strip())\n",
    "        cycloneNum.append(fields[7].strip())\n",
    "        year.append(fields[8].strip())\n",
    "        stormType.append(fields[9].strip())\n",
    "        filename.append(fields[-1].strip().lower())\n",
    "\n",
    "    storms = DataFrame({'Name': name, 'Basin': basin, 'CycloneNum': np.array(cycloneNum),\n",
    "                        'Year': np.array(year), 'StormType': stormType,\n",
    "                        'Filename': filename})\n",
    "    return(storms)"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 6,
   "metadata": {},
   "outputs": [],
   "source": [
    "class Storm_Selection_gui:\n",
    "    \"\"\"Storm_Selection_gui is a graphic user interface object designed for selecting storms\n",
    "    from the National Hurricane Center's storm list database.\"\"\"\n",
    "\n",
    "    def __init__(self):\n",
    "        \"\"\"__init__ is the initiation function for the Storm_Selection_gui object. This\n",
    "        initiation creates a dataframe for the storm data from the National Hurricane\n",
    "        Center. In addition, this initiation creates widgets to select a specific storm\n",
    "        from the National Hurricane Center database with both a widget for a year\n",
    "        selection and also for a track button which actually retrieves the models\n",
    "        and tracks for a given storm. \"\"\"\n",
    "\n",
    "        # Setting up storm object table\n",
    "        fileLines = readUrlFile('http://ftp.nhc.noaa.gov/atcf/index/storm_list.txt')\n",
    "        self.storm_table = split_storm_info(fileLines)\n",
    "\n",
    "        # Creation of widgets\n",
    "        # Year Selector Slider (year_slider)\n",
    "        self.year_slider = widgets.IntSlider(min=1851, max=2019, value=2019,\n",
    "                                             description='Storm Year: ')\n",
    "        widgets.interact(self.get_storms_slider, year_slider=self.year_slider)\n",
    "        # Button to retrieve storm tracks (track_button)\n",
    "        self.track_button = widgets.ToggleButton(value=False, description='Get Storm Tracks',\n",
    "                                                 disabled=False, button_style='info',\n",
    "                                                 tooltip='Description')\n",
    "        widgets.interact(self.get_tracks, track_button=self.track_button)\n",
    "\n",
    "    def get_storms_slider(self, year_slider):\n",
    "        \"\"\"get_storms_slider is a function that is written to take a user defined year from\n",
    "        1851 to 2019 and then trim the split_storms_info dataframe to just have\n",
    "        storms from that year. This takes in the interactive widget called year_slider.\"\"\"\n",
    "\n",
    "        self.one_year_table = self.storm_table[self.storm_table.Year == str(year_slider)]\n",
    "        self.storm_names = widgets.Dropdown(options=self.one_year_table['Name'],\n",
    "                                            description='Storm Names: ')\n",
    "        widgets.interact(self.get_name_dropdown, storm_names=self.storm_names)\n",
    "\n",
    "    def get_name_dropdown(self, storm_names):\n",
    "        \"\"\"get_name_dropdown is a function that allows for selection of a specific storm\n",
    "        based on its name. This function returns the filename of a given storm. This\n",
    "        function interacts with the widget called forecast_select.\"\"\"\n",
    "\n",
    "        name = self.storm_names.value\n",
    "        one_storm_row = self.one_year_table[self.one_year_table.Name == name]\n",
    "        self.filename = one_storm_row.Filename\n",
    "        file_name = self.filename.tolist()\n",
    "        if self.filename.empty is False:\n",
    "            self.filename = file_name[0]\n",
    "        elif self.filename.empty is True:\n",
    "            print('No file name data for this year.')\n",
    "\n",
    "    def get_models(self, models):\n",
    "        \"\"\"get_models is a function that is linked to the selection widget which allows for\n",
    "        multiple models to be selected to plot for a specific hurricane track.\"\"\"\n",
    "\n",
    "        self.model_selection_latlon(models)\n",
    "\n",
    "    def time_slider(self, time_slide):\n",
    "        \"\"\"time_slider is a function which changes the time for which to plot the models for\n",
    "        a given hurricane track.\"\"\"\n",
    "\n",
    "        time = self.date_times[time_slide]\n",
    "        return(time)\n",
    "\n",
    "    def get_tracks(self, track_button):\n",
    "        \"\"\"get_tracks is a function that will create the url and pull the data for either\n",
    "        the forecast track or best track for a given storm. The Url is made by using both\n",
    "        the year and the filename. This function will then read the data and create a data\n",
    "        frame for both the forecast and best tracks and compile these data frames into a\n",
    "        dictionary. This function returns this dictionary of forecast and best tracks. \"\"\"\n",
    "\n",
    "        year = str(self.year_slider.value)\n",
    "        filename = self.filename\n",
    "        data_dictionary = {}\n",
    "        # Current year data is stored in a different location\n",
    "        if year == '2019':\n",
    "            urlf = 'http://ftp.nhc.noaa.gov/atcf/aid_public/a{}.dat.gz'.format(filename)\n",
    "            urlb = 'http://ftp.nhc.noaa.gov/atcf/btk/b{}.dat'.format(filename)\n",
    "        else:\n",
    "            urlf = 'http://ftp.nhc.noaa.gov/atcf/archive/{}/a{}.dat.gz'.format(year, filename)\n",
    "            urlb = 'http://ftp.nhc.noaa.gov/atcf/archive/{}/b{}.dat.gz'.format(year, filename)\n",
    "\n",
    "        url_links = [urlf, urlb]\n",
    "        url_count = 0\n",
    "        if track_button is True:\n",
    "            for url in url_links:\n",
    "                # Checking if url is valid, if status_code is 200 then website is active\n",
    "                if requests.get(url).status_code == 200:\n",
    "                    if url.endswith('.dat'):\n",
    "                        lines = readUrlFile(url)\n",
    "                    else:\n",
    "                        lines = readGZFile(url)\n",
    "\n",
    "                    # Splitting the method for which we will create the dataframe\n",
    "                    lat, lon, basin, cycloneNum = [], [], [], []\n",
    "                    warnDT, model, forecast_hour = [], [], []\n",
    "                    for line in lines:\n",
    "                        line = str(line)\n",
    "                        line = line[2:]\n",
    "                        fields = line.split(',')\n",
    "                        # Joins together lattitude and longitude strings without\n",
    "                        # directional letters.\n",
    "                        # Includes type conversion in order to divide by 10 to\n",
    "                        # get the correct coordinate.\n",
    "                        latSingle = int(fields[6][:-1])/10.0\n",
    "                        lonSingle = -(int(fields[7][:-1])/10.0)\n",
    "                        lat.append(latSingle)\n",
    "                        lon.append(lonSingle)\n",
    "                        basin.append(fields[0])\n",
    "                        forecast_hour.append(fields[5])\n",
    "                        cycloneNum.append(fields[1].strip())\n",
    "                        warnDT.append(fields[2].strip())\n",
    "                        model.append(fields[4].strip())\n",
    "\n",
    "                        # Combining data from file into a Pandas Dataframe.\n",
    "                        storm_data_frame = DataFrame({'Basin': basin,\n",
    "                                                      'CycloneNum': np.array(cycloneNum),\n",
    "                                                      'WarnDT': np.array(warnDT),\n",
    "                                                      'Model': model, 'Lat': np.array(lat),\n",
    "                                                      'Lon': np.array(lon),\n",
    "                                                      'forecast_hour':\n",
    "                                                      np.array(forecast_hour)})\n",
    "                        # Adding this newly created DataFrame to a dictionary\n",
    "                        if url_count == 0:\n",
    "                            data_dictionary['forecast'] = storm_data_frame\n",
    "                        else:\n",
    "                            data_dictionary['best_track'] = storm_data_frame\n",
    "\n",
    "                else:\n",
    "                    print('url {} was not valid, select different storm.'.format(url))\n",
    "                    track_button = False\n",
    "\n",
    "                url_count += 1\n",
    "\n",
    "            self.storm_dictionary = data_dictionary\n",
    "            forecast = data_dictionary.get('forecast')\n",
    "\n",
    "            unique_models, unique_index = list(np.unique(forecast['Model'].values,\n",
    "                                               return_index=True))\n",
    "\n",
    "            # Selection tool to pick from available models to plot (model_select)\n",
    "            self.model_select = widgets.SelectMultiple(options=unique_models,\n",
    "                                                       value=[unique_models[0]],\n",
    "                                                       description='Models: ',\n",
    "                                                       disabled=False)\n",
    "            widgets.interact(self.get_models, models=self.model_select)\n",
    "\n",
    "    def model_selection_latlon(self, models):\n",
    "        \"\"\"model_selection_latlon is a function that allows the user to select a model for\n",
    "        a given storm and whether the tracks are forecast or best tracks. The parameters\n",
    "        for this are a string stating whether the user wants forecast or best tracks and\n",
    "        also all model outputs for all forecasts and best tracks compiled into a python\n",
    "        dictionary. The latlon part of this function comes from taking the users selected\n",
    "        model and getting the latitudes and longitudes of all positions of the storm for\n",
    "        this forecast. This function then returns these lats and lons as a pandas.Series\"\"\"\n",
    "\n",
    "        if self.model_select.disabled is False:\n",
    "            # We will always plot best track, and thus must save the coordinates for plotting\n",
    "            best_track = self.storm_dictionary.get('best_track')\n",
    "            self.date_times = best_track['WarnDT']\n",
    "            lats = best_track['Lat']\n",
    "            lons = best_track['Lon']\n",
    "            self.best_track_coordinates = [lats, lons]\n",
    "\n",
    "            model_tracks = self.storm_dictionary.get('forecast')\n",
    "\n",
    "            self.model_table = []\n",
    "            for model in models:\n",
    "                one_model_table = model_tracks[model_tracks['Model'] == model]\n",
    "                self.model_table.append(one_model_table)\n",
    "\n",
    "            # Slider to choose time frame for which to plot models (plot_slider)\n",
    "            self.plot_slider = widgets.IntSlider(min=0, max=(len(self.date_times)-1),\n",
    "                                                 value=0, description='Tracks Time',\n",
    "                                                 disabled=False)\n",
    "            widgets.interact(self.plotting, plot_slider=self.plot_slider)\n",
    "\n",
    "    def plotting(self, plot_slider):\n",
    "        \"\"\"plotting is a function that that plots the models tracks and best tracks for all\n",
    "        selected models. These tracks are then plotted on the Blue Marble projection. \"\"\"\n",
    "\n",
    "        if self.plot_slider.disabled is False:\n",
    "            # Identifying the time associated with the models for time text box\n",
    "            year = self.date_times[plot_slider][0: 4]\n",
    "            month = self.date_times[plot_slider][4: 6]\n",
    "            day = self.date_times[plot_slider][6: 8]\n",
    "            hour = self.date_times[plot_slider][8: 10]\n",
    "            time_string = 'Date: {0}/{1}/{2} \\n Hour: {3}'.format(month, day, year, hour)\n",
    "\n",
    "            # Finding data for best track, and extremes for which to base axis extent on\n",
    "            self.best_lats = np.array(self.best_track_coordinates[0])\n",
    "            self.best_lons = np.array(self.best_track_coordinates[1])\n",
    "            min_best_lat = min(self.best_lats)\n",
    "            max_best_lat = max(self.best_lats)\n",
    "            min_best_lon = min(self.best_lons)\n",
    "            max_best_lon = max(self.best_lons)\n",
    "\n",
    "            # Plotting the track on a cartopy stock image\n",
    "            self.fig = plt.figure(figsize=(14, 11))\n",
    "            self.ax = self.fig.add_subplot(1, 1, 1, projection=ccrs.PlateCarree())\n",
    "            self.ax.stock_img()\n",
    "            self.data_projection = ccrs.PlateCarree()\n",
    "            self.ax.plot(self.best_lons, self.best_lats, marker='o', color='white',\n",
    "                         label='Best Track', transform=self.data_projection)\n",
    "            self.ax.set_extent([(min_best_lon - 30), (max_best_lon + 30),\n",
    "                                (min_best_lat - 30), (max_best_lat + 30)])\n",
    "            jet = plt.get_cmap('jet')\n",
    "            colors = iter(jet(np.linspace(0.2, 1, (len(self.model_select.value)+1))))\n",
    "            left = .1\n",
    "            bottom = .1\n",
    "            self.ax.text(left, bottom, time_string, transform=self.ax.transAxes,\n",
    "                         fontsize=14, color='black')\n",
    "            for model_type in self.model_table:\n",
    "                one_model_time = model_type[model_type['WarnDT'] ==\n",
    "                                            self.date_times[plot_slider]]\n",
    "                lats = one_model_time['Lat'].tolist()\n",
    "                lons = one_model_time['Lon'].tolist()\n",
    "                if len(lats) != 0:\n",
    "                    model_list = model_type['Model'].tolist()\n",
    "                    self.ax.plot(lons, lats, marker='o', color=next(colors),\n",
    "                                 label=model_list[0])\n",
    "            plt.title('Storm Name: {0} Year: {1}'.format(self.storm_names.value,\n",
    "                      str(self.year_slider.value)))\n",
    "            plt.legend()"
   ]
  },
  {
   "cell_type": "code",
   "execution_count": 7,
   "metadata": {},
   "outputs": [
    {
     "data": {
      "application/vnd.jupyter.widget-view+json": {
       "model_id": "a572e7f3d96d49ab805f2231d040a19d",
       "version_major": 2,
       "version_minor": 0
      },
      "text/plain": [
       "interactive(children=(ToggleButton(value=False, button_style='info', description='Get Storm Tracks', tooltip='…"
      ]
     },
     "metadata": {},
     "output_type": "display_data"
    }
   ],
   "source": [
    "######################################################################\n",
    "storm_selection = Storm_Selection_gui()"
   ]
  }
 ],
 "metadata": {
  "jupytext": {
   "cell_metadata_filter": "-all",
   "main_language": "python",
   "notebook_metadata_filter": "-all"
  },
  "kernelspec": {
   "display_name": "Python 3",
   "language": "python",
   "name": "python3"
  },
  "language_info": {
   "codemirror_mode": {
    "name": "ipython",
    "version": 3
   },
   "file_extension": ".py",
   "mimetype": "text/x-python",
   "name": "python",
   "nbconvert_exporter": "python",
   "pygments_lexer": "ipython3",
   "version": "3.7.8"
  },
  "widgets": {
   "application/vnd.jupyter.widget-state+json": {
    "state": {
     "0698391af24f46e28952b573ff4ce228": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "DropdownModel",
      "state": {
       "_dom_classes": [],
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "DropdownModel",
       "_options_labels": [
        "ANDREA",
        "BARRY",
        "THREE",
        "CHANTAL",
        "DORIAN",
        "ERIN",
        "FERNAND",
        "GABRIELLE",
        "NINE",
        "JERRY",
        "IMELDA",
        "KAREN",
        "LORENZO",
        "MELISSA",
        "FIFTEEN",
        "SIXTEEN",
        "OLGA",
        "PABLO",
        "REBEKAH",
        "SEBASTIEN",
        "GENESIS018",
        "GENESIS019",
        "GENESIS021",
        "GENESIS022",
        "INVEST",
        "INVEST",
        "EMA",
        "ALVIN",
        "BARBARA",
        "COSME",
        "FOUR",
        "DALILA",
        "ERICK",
        "FLOSSIE",
        "GIL",
        "HENRIETTE",
        "IVO",
        "JULIETTE",
        "AKONI",
        "KIKO",
        "MARIO",
        "LORENA",
        "NARDA",
        "SEVENTEEN",
        "OCTAVE",
        "PRISCILLA",
        "RAYMOND",
        "TWENTY-ONE",
        "GENESIS029",
        "GENESIS030",
        "GENESIS031",
        "IBA"
       ],
       "_view_count": null,
       "_view_module": "@jupyter-widgets/controls",
       "_view_module_version": "1.5.0",
       "_view_name": "DropdownView",
       "description": "Storm Names: ",
       "description_tooltip": null,
       "disabled": false,
       "index": 0,
       "layout": "IPY_MODEL_f2915eebb6df4335a79624d01919cee3",
       "style": "IPY_MODEL_f9baf851b7404f82aaa5cf5506197932"
      }
     },
     "23d203d6129c43c6ba363ecfc8321bb1": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "VBoxModel",
      "state": {
       "_dom_classes": [
        "widget-interact"
       ],
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "VBoxModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/controls",
       "_view_module_version": "1.5.0",
       "_view_name": "VBoxView",
       "box_style": "",
       "children": [
        "IPY_MODEL_3cdd3f4e16ad48b9ad58c8f08f0323cc",
        "IPY_MODEL_73c255aebc9e484ab1f1d25b6e24abde"
       ],
       "layout": "IPY_MODEL_f23463749fa945bd965a320c98661c61"
      }
     },
     "2851064cb4f4472c9102b96005cd50f3": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "VBoxModel",
      "state": {
       "_dom_classes": [
        "widget-interact"
       ],
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "VBoxModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/controls",
       "_view_module_version": "1.5.0",
       "_view_name": "VBoxView",
       "box_style": "",
       "children": [
        "IPY_MODEL_0698391af24f46e28952b573ff4ce228",
        "IPY_MODEL_9731782d26454fc29f3724dd1175ad5c"
       ],
       "layout": "IPY_MODEL_cdf29472a3ef46feb06f4520bff3c325"
      }
     },
     "312cdd4f85b74effa67c6c268a0d7066": {
      "model_module": "@jupyter-widgets/output",
      "model_module_version": "1.0.0",
      "model_name": "OutputModel",
      "state": {
       "_dom_classes": [],
       "_model_module": "@jupyter-widgets/output",
       "_model_module_version": "1.0.0",
       "_model_name": "OutputModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/output",
       "_view_module_version": "1.0.0",
       "_view_name": "OutputView",
       "layout": "IPY_MODEL_46868612be31466687e50229ec844dd3",
       "msg_id": "",
       "outputs": []
      }
     },
     "3cdd3f4e16ad48b9ad58c8f08f0323cc": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "IntSliderModel",
      "state": {
       "_dom_classes": [],
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "IntSliderModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/controls",
       "_view_module_version": "1.5.0",
       "_view_name": "IntSliderView",
       "continuous_update": true,
       "description": "Storm Year: ",
       "description_tooltip": null,
       "disabled": false,
       "layout": "IPY_MODEL_e9a72b19641a4233b110b71617cd9f57",
       "max": 2019,
       "min": 1851,
       "orientation": "horizontal",
       "readout": true,
       "readout_format": "d",
       "step": 1,
       "style": "IPY_MODEL_f20139bba338446f9e85e82781a4fc8f",
       "value": 2019
      }
     },
     "46868612be31466687e50229ec844dd3": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "73c255aebc9e484ab1f1d25b6e24abde": {
      "model_module": "@jupyter-widgets/output",
      "model_module_version": "1.0.0",
      "model_name": "OutputModel",
      "state": {
       "_dom_classes": [],
       "_model_module": "@jupyter-widgets/output",
       "_model_module_version": "1.0.0",
       "_model_name": "OutputModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/output",
       "_view_module_version": "1.0.0",
       "_view_name": "OutputView",
       "layout": "IPY_MODEL_a5ccccb39eae4ea8b9271ac889b2e77f",
       "msg_id": "",
       "outputs": []
      }
     },
     "904a8387fa734c51b7fa485c76e8918f": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "DescriptionStyleModel",
      "state": {
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "DescriptionStyleModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "StyleView",
       "description_width": ""
      }
     },
     "91490589a2e84c1f8b2fb56797af4b12": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "ToggleButtonModel",
      "state": {
       "_dom_classes": [],
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "ToggleButtonModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/controls",
       "_view_module_version": "1.5.0",
       "_view_name": "ToggleButtonView",
       "button_style": "info",
       "description": "Get Storm Tracks",
       "description_tooltip": null,
       "disabled": false,
       "icon": "",
       "layout": "IPY_MODEL_afae65d1ed6c4b3f8055292804013625",
       "style": "IPY_MODEL_904a8387fa734c51b7fa485c76e8918f",
       "tooltip": "Description",
       "value": false
      }
     },
     "9731782d26454fc29f3724dd1175ad5c": {
      "model_module": "@jupyter-widgets/output",
      "model_module_version": "1.0.0",
      "model_name": "OutputModel",
      "state": {
       "_dom_classes": [],
       "_model_module": "@jupyter-widgets/output",
       "_model_module_version": "1.0.0",
       "_model_name": "OutputModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/output",
       "_view_module_version": "1.0.0",
       "_view_name": "OutputView",
       "layout": "IPY_MODEL_aa76fa5f2f2a4d9a81ba5d76ce0a8034",
       "msg_id": "",
       "outputs": []
      }
     },
     "a572e7f3d96d49ab805f2231d040a19d": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "VBoxModel",
      "state": {
       "_dom_classes": [
        "widget-interact"
       ],
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "VBoxModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/controls",
       "_view_module_version": "1.5.0",
       "_view_name": "VBoxView",
       "box_style": "",
       "children": [
        "IPY_MODEL_91490589a2e84c1f8b2fb56797af4b12",
        "IPY_MODEL_312cdd4f85b74effa67c6c268a0d7066"
       ],
       "layout": "IPY_MODEL_e5475e29cd964a6fb7305be5a9dc7cde"
      }
     },
     "a5ccccb39eae4ea8b9271ac889b2e77f": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "aa76fa5f2f2a4d9a81ba5d76ce0a8034": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "afae65d1ed6c4b3f8055292804013625": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "cdf29472a3ef46feb06f4520bff3c325": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "e5475e29cd964a6fb7305be5a9dc7cde": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "e9a72b19641a4233b110b71617cd9f57": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "f20139bba338446f9e85e82781a4fc8f": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "SliderStyleModel",
      "state": {
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "SliderStyleModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "StyleView",
       "description_width": "",
       "handle_color": null
      }
     },
     "f23463749fa945bd965a320c98661c61": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "f2915eebb6df4335a79624d01919cee3": {
      "model_module": "@jupyter-widgets/base",
      "model_module_version": "1.2.0",
      "model_name": "LayoutModel",
      "state": {
       "_model_module": "@jupyter-widgets/base",
       "_model_module_version": "1.2.0",
       "_model_name": "LayoutModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "LayoutView",
       "align_content": null,
       "align_items": null,
       "align_self": null,
       "border": null,
       "bottom": null,
       "display": null,
       "flex": null,
       "flex_flow": null,
       "grid_area": null,
       "grid_auto_columns": null,
       "grid_auto_flow": null,
       "grid_auto_rows": null,
       "grid_column": null,
       "grid_gap": null,
       "grid_row": null,
       "grid_template_areas": null,
       "grid_template_columns": null,
       "grid_template_rows": null,
       "height": null,
       "justify_content": null,
       "justify_items": null,
       "left": null,
       "margin": null,
       "max_height": null,
       "max_width": null,
       "min_height": null,
       "min_width": null,
       "object_fit": null,
       "object_position": null,
       "order": null,
       "overflow": null,
       "overflow_x": null,
       "overflow_y": null,
       "padding": null,
       "right": null,
       "top": null,
       "visibility": null,
       "width": null
      }
     },
     "f9baf851b7404f82aaa5cf5506197932": {
      "model_module": "@jupyter-widgets/controls",
      "model_module_version": "1.5.0",
      "model_name": "DescriptionStyleModel",
      "state": {
       "_model_module": "@jupyter-widgets/controls",
       "_model_module_version": "1.5.0",
       "_model_name": "DescriptionStyleModel",
       "_view_count": null,
       "_view_module": "@jupyter-widgets/base",
       "_view_module_version": "1.2.0",
       "_view_name": "StyleView",
       "description_width": ""
      }
     }
    },
    "version_major": 2,
    "version_minor": 0
   }
  }
 },
 "nbformat": 4,
 "nbformat_minor": 4
}
